package org.bitbucket.bradleysmithllc.etlagent.informatica;

import com.fasterxml.jackson.databind.JsonNode;
import org.bitbucket.bradleysmithllc.etlagent.AbstractAgent;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.InformaticaConfiguration;
import org.bitbucket.bradleysmithllc.etlagent.informatica.provider.JsonFeatureModule;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.feature.FeatureOperation;
import org.bitbucket.bradleysmithllc.etlunit.feature.ResourceFeatureMetaInfo;

import java.util.*;

public class InformaticaAgent extends AbstractAgent
{
	private JsonNode configurationNode;
	private InformaticaConfiguration informaticaConfiguration;
	private final ResourceFeatureMetaInfo resourceFeatureMetaInfo;

	private RuntimeSupport runtimeSupport;

	public InformaticaAgent() {
		resourceFeatureMetaInfo = new ResourceFeatureMetaInfo(new JsonFeatureModule());
	}

	@Override
	public void receiveRuntimeSupport(RuntimeSupport runtimeSupport) {
		this.runtimeSupport = runtimeSupport;
	}

	@Override
	public String getId() {
		return "informatica";
	}

	@Override
	public String getVersion() {
		return "1.0";
	}

	@Override
	public JsonNode getConfigurationValidator() {
		return resourceFeatureMetaInfo.getFeatureConfigurationValidatorNode();
	}

	@Override
	public void setAgentConfiguration(JsonNode jsonNode) {
		configurationNode = jsonNode;

		informaticaConfiguration = new InformaticaConfiguration(jsonNode, runtimeSupport, null);
	}

	@Override
	public List<RequestHandler> getRequestHandlers() {
		List<RequestHandler> handlers = new ArrayList<RequestHandler>();

		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		ServiceLoader<InformaticaRequestHandler> sl = ServiceLoader.load(InformaticaRequestHandler.class, cl);

		Iterator<InformaticaRequestHandler> it = sl.iterator();

		while (it.hasNext())
		{
			InformaticaRequestHandler informaticaRequestHandler = it.next();

			Map<String,FeatureOperation> operationMap = resourceFeatureMetaInfo.getExportedOperations();

			if (!operationMap.containsKey(informaticaRequestHandler.getId()))
			{
				throw new IllegalArgumentException("Informatica operation not found: " + informaticaRequestHandler.getId());
			}

			informaticaRequestHandler.setInformaticaConfiguration(informaticaConfiguration);
			informaticaRequestHandler.receiveRuntimeSupport(runtimeSupport);
			informaticaRequestHandler.setFeatureOperation(operationMap.get(informaticaRequestHandler.getId()));

			handlers.add(informaticaRequestHandler);
		}

		return handlers;
	}
}