package org.bitbucket.bradleysmithllc.etlagent.informatica.provider;

import com.google.gson.stream.JsonWriter;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.bitbucket.bradleysmithllc.etlagent.informatica.dto.RemoteFile;
import org.bitbucket.bradleysmithllc.etlagent.resources.ContextBase;
import org.bitbucket.bradleysmithllc.etlunit.RuntimeSupport;
import org.bitbucket.bradleysmithllc.etlunit.io.FileBuilder;
import org.bitbucket.bradleysmithllc.webserviceshubclient.JCmd;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;

import java.io.File;
import java.io.StringWriter;
import java.util.*;

public class InformaticaWebServicesHubClientImpl implements InformaticaWebServicesHubClient {
	private final InformaticaWebServicesHub informaticaWebServicesHub;
	private final InformaticaIntegrationService informaticaIntegrationService;
	private final RuntimeSupport runtimeSupport;

	public InformaticaWebServicesHubClientImpl(InformaticaWebServicesHub informaticaWebServicesHub, InformaticaIntegrationService informaticaIntegrationService, RuntimeSupport runtimeSupport) {
		this.informaticaWebServicesHub = informaticaWebServicesHub;
		this.informaticaIntegrationService = informaticaIntegrationService;
		this.runtimeSupport = runtimeSupport;
	}

	@Override
	public InformaticaWebServicesHub getWebServicesHub() {
		return informaticaWebServicesHub;
	}

	@Override
	public InformaticaDomain getInformaticaDomain() {
		return informaticaWebServicesHub.getInformaticaDomain();
	}

	@Override
	public void connect() throws Exception {
	}

	@Override
	public void dispose() {
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, null, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflow(String workFlow, String runInstanceName, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, null, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		return executeWorkflowTask(workFlow, null, task, folder, parmFile, remoteFiles);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile) throws Exception {
		return executeWorkflowTask(workFlow, runInstanceName, task, folder, parmFile, null);
	}

	@Override
	public InformaticaExecutionResult executeWorkflowTask(String workFlow, String runInstanceName, String task, String folder, ParameterFile parmFile, Map<RemoteFile.fileType, List<RemoteFile>> remoteFiles) throws Exception {
		if (remoteFiles != null) {
			// lay out the input files
			RemoteFile.copyRemoteToLocal(remoteFiles, getInformaticaDomain());
		}

		List<String> args = new ArrayList<String>();

		args.add("--debug");
		args.add("--verbose");

		if (runInstanceName != null) {
			args.add("--run-instance");
			args.add(runInstanceName);
		}

		String uuid = UUID.randomUUID().toString();

		File par = new File(
				getInformaticaDomain().getParameterFilesDir(),
				uuid + ".PRM"
		);

		parmFile.write(par);

		args.add("--parameter-file-path");
		args.add(par.getAbsolutePath());

		if (getInformaticaDomain().getSecurityDomain() != null) {
			args.add("--security-domain");
			args.add(getInformaticaDomain().getSecurityDomain());
		}

		args.add("--domain-name");
		args.add(getInformaticaDomain().getDomainName());

		args.add("--repository-name");
		args.add(informaticaWebServicesHub.getInformaticaRepository().getRepositoryName());

		args.add("--integration-service");
		args.add(informaticaIntegrationService.getIntegrationServiceName());

		args.add("--user-name");
		args.add(getInformaticaDomain().getUserName());

		args.add("--password");
		String password = getInformaticaDomain().getPassword();

		if (password == null) {
			throw new IllegalArgumentException("Web services hub requires a plain-text password");
		}

		args.add(password);

		args.add("--wsdl-url");
		args.add("http://" + informaticaWebServicesHub.getHostName() + ":" + informaticaWebServicesHub.getPort() + "/wsh/services/BatchServices/DataIntegration?WSDL");

		args.add("--folder");
		args.add(folder);

		args.add("--workflow");
		args.add(workFlow);

		if (task != null) {
			args.add("--task-instance-path");
			args.add(task);
		}

		// save the web service call into a json file
		File wshRoot = new FileBuilder(runtimeSupport.getGeneratedSourceDirectory("web-services-hub")).subdir(informaticaWebServicesHub.getName()).name(uuid + ".json").file();

		StringWriter stw = new StringWriter();
		JsonWriter jwr = new JsonWriter(stw);

		try {
			jwr.beginObject();
			jwr.name("jcmd-arguments");
			jwr.beginArray();

			for (String arg : args) {
				jwr.value(arg);
			}

			jwr.endArray();
			jwr.endObject();
		} finally {
			jwr.close();
		}

		FileUtils.write(wshRoot, stw.toString());

		Map<RemoteFile.fileType, List<RemoteFile>> xfiles = new HashMap<RemoteFile.fileType, List<RemoteFile>>();
		String excStr = null;

		try {
			if (!ContextBase.agentTestMode()) {
				JCmd jcmd = new JCmd();

				jcmd.main(args.toArray(new String[args.size()]));
			}
		} catch (Exception exc) {
			excStr = exc.toString();
		} finally {
			RemoteFile.copyLocalToRemote(getInformaticaDomain(), xfiles);
		}

		return new InformaticaExecutionResult(xfiles, excStr);
	}
}